//
//  Interface.cpp
//  Lead Singer
//
//  Created by Alvin Heng on 12/4/11.
//  Copyright 2011 Stanford University. All rights reserved.
//

#include <iostream>
#include "Interface.h"
#include "include/fluidsynth.h"
#include <math.h>

using namespace std;

UIButton* _clickedButton;
UIButton* _draggedButton;
STPoint2 _mouseDragStart;
UIButton* _topButton = NULL;
bool _rotate = false;
void playbackAll();
void playback();
void startStopRec();
void ResetOutfile();
void ChangeMode(SynthMode currMode);
void Exit();

Interface* _currentInstance;

#pragma mark - Constructor
Interface::Interface() {
    _winWidth = 512;
    _winHeight = 512;
    
    _bKeyHeight = 40.0f;
    _wKeyHeight = _bKeyHeight * 1.5f;
    
    _rRoomSize = FLUID_REVERB_DEFAULT_ROOMSIZE;
    _rDamp = FLUID_REVERB_DEFAULT_DAMP;
    _rWidth = 0.0f;//FLUID_REVERB_DEFAULT_WIDTH;
    _rLevel = FLUID_REVERB_DEFAULT_LEVEL;
    
    _rBoxWidth = _winWidth;
    _rBoxHeight = _winHeight - _topBarHeight - _keyboardLength;
    _rSize = (float)_winWidth * 0.25f * 0.25f;
    _rX = _rRoomSize * _rBoxWidth / MAX_RROOMSIZE;
    _rY = _rBoxHeight * _rDamp / MAX_RDAMP;
    _rRotate = _rWidth * 360.0f / MAX_RWIDTH;
    
    _topBarHeight = 40.0f;
    _symbolSize = 30.0f;
    _symbolGap = 5.0f;
    _currentMode = PIANOMODE;
    
    Screen pScreen, gScreen;
    pScreen.numBeats = 0;
    gScreen.numBeats = 0;
    pScreen.translationOffset = 0.0f;
    gScreen.translationOffset = 0.0f;
    _beatScreen[0] = pScreen;
    _beatScreen[1] = gScreen;
    _screenDrag = false;
    _isRecording = false;
}

Interface::~Interface() {
}
#pragma mark -

#pragma mark - Beat Screen
void Interface::drawCursor() {
}

void Interface::drawBeat(Key key) {
    glPushMatrix();
    glTranslatef(key.x, key.y, 0.0f);
    if (!key.doneAnimating) {
        glTranslatef(0.0f, 0.0f, 1.0f);
    }
    
    float keyWidth = (key.whiteKey) ? (_wKeyWidth - (0.5 * _bKeyWidth)) : _bKeyWidth;
    
    glColor3f(1.0f, 1.0f, 1.0f);
    glRectf(0.0f, 0.0f, KEY_HEIGHT * key.duration, keyWidth);
    
    glColor3f(0.0f, 0.0f, 0.0f);
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glRectf(0.0f, 0.0f, KEY_HEIGHT * key.duration, keyWidth);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    
    glPopMatrix();
}

void Interface::drawCurrentScreen() {
    glPushMatrix();
    glTranslatef(-_beatScreen[_currentMode].translationOffset, -_topBarHeight - 1.0f, 0.0f);
    int size = _beatScreen[_currentMode].numBeats;
    Key* screen = _beatScreen[_currentMode].beats;
    for (int i = 0 ; i < size; i++) {
        Key currKey = screen[i];
        float endX = currKey.x;
        if (!currKey.doneAnimating) {
            currKey.x = currKey.timer * endX;
            currKey.timer += 0.001f;
            if (currKey.timer >= 1.0f) {
                currKey.doneAnimating = true;
            }
        } 
        drawBeat(currKey);
        currKey.x = endX;
        screen[i] = currKey;
    }
    glPopMatrix();
}

void Interface::extendBeat() {
    if (_isRecording) {
        Screen currScreen = _beatScreen[_currentMode];
        _beatScreen[_currentMode].beats[currScreen.numBeats-1].duration += 0.1f;
        float duration = _beatScreen[_currentMode].beats[currScreen.numBeats-1].duration;
        float endX = currScreen.beats[currScreen.numBeats-1].x + (KEY_HEIGHT * duration);
        if (endX > _winWidth) {
            _beatScreen[_currentMode].translationOffset = endX - _winWidth;
        }   
    }
}

void Interface::addBeat(int noteIndex, char note) {
    if (_isRecording) {
        Key newKey = _keyboard[noteIndex - (NUM_KEYS * FIRST_OCTAVE)];
        newKey.noteIndex = noteIndex;
        newKey.duration = 1.0f;
        Screen currScreen = _beatScreen[_currentMode];
        if (currScreen.numBeats == 0) {
            newKey.x = 1.5f * KEY_HEIGHT;
        } else {
            Key oldKey = currScreen.beats[currScreen.numBeats - 1];
            newKey.x = oldKey.x + oldKey.duration * KEY_HEIGHT + 1.0f;
        }
        //Beats moving off screen
        if (newKey.x + KEY_HEIGHT > _winWidth) {
            _beatScreen[_currentMode].translationOffset = newKey.x + KEY_HEIGHT - _winWidth;
        }
        _beatScreen[_currentMode].beats[_beatScreen[_currentMode].numBeats++] = newKey;
    }
}

bool Interface::screenIsHit(STPoint2 m) {
    return (m.x > 1.5f * KEY_HEIGHT && m.x < _winWidth &&
            m.y > _winHeight - _topBarHeight - _keyboardLength && 
            m.y < _winHeight - _topBarHeight);
}
#pragma mark -

#pragma mark - Top Bar
void Interface::GuitarMode() {
    if (_currentMode != GUITARMODE) {
        if (fluid_synth_sfload(_fsynth, "d_guitar.sf2", 1) == -1) {
            cerr << "Error loading sound font" << endl;
            exit(1);
        }
        _currentMode = GUITARMODE;
        _gButton->SetColor(SELECTED_BUTTON_COLOR);
        _pButton->SetColor(TOP_BUTTON_COLOR);
        _sButton->SetColor(TOP_BUTTON_COLOR);
        _dButton->SetColor(TOP_BUTTON_COLOR);
        ChangeMode(GUITARMODE);
    }
}

void Interface::PianoMode() {
    if (_currentMode != PIANOMODE) {
        if (fluid_synth_sfload(_fsynth, "piano.sf2", 1) == -1) {
            cerr << "Error loading sound font" << endl;
            exit(1);
        }
        _currentMode = PIANOMODE;
        _pButton->SetColor(SELECTED_BUTTON_COLOR);
        _gButton->SetColor(TOP_BUTTON_COLOR);
        _sButton->SetColor(TOP_BUTTON_COLOR);
        _dButton->SetColor(TOP_BUTTON_COLOR);
        ChangeMode(PIANOMODE);
    }
}

void Interface::DrumMode() {
    if (_currentMode != DRUMMODE) {
        if (fluid_synth_sfload(_fsynth, "drum.sf2", 1) == -1) {
            cerr << "Error loading sound font" << endl;
            exit(1);
        }
        _currentMode = DRUMMODE;
        _dButton->SetColor(SELECTED_BUTTON_COLOR);
        _pButton->SetColor(TOP_BUTTON_COLOR);
        _sButton->SetColor(TOP_BUTTON_COLOR);
        _gButton->SetColor(TOP_BUTTON_COLOR);
        ChangeMode(DRUMMODE);
    }
}

void Interface::SynthMode() {
    if (_currentMode != SYNTHMODE) {
        if (fluid_synth_sfload(_fsynth, "synth.sf2", 1) == -1) {
            cerr << "Error loading sound font" << endl;
            exit(1);
        }
        _currentMode = SYNTHMODE;
        _sButton->SetColor(SELECTED_BUTTON_COLOR);
        _pButton->SetColor(TOP_BUTTON_COLOR);
        _gButton->SetColor(TOP_BUTTON_COLOR);
        _dButton->SetColor(TOP_BUTTON_COLOR);
        ChangeMode(SYNTHMODE);
    }
}

void Interface::PlayAll() {
    if (!_isRecording) {
        playbackAll();
    }
}

void Interface::PlayCurrent() {
    if (!_isRecording) {
        playback();
        _isPlayingBack = true;
    }
}

void Interface::Reset() {
    _beatScreen[_currentMode].numBeats = 0;
    _beatScreen[_currentMode].translationOffset = 0;
    ResetOutfile();
}

void Interface::Record() {
    _isRecording = !_isRecording;
    startStopRec();
}

void GuitarModeCallback(UIButton* button) {
    _currentInstance->GuitarMode();
}

void PianoModeCallback(UIButton* button) {
    _currentInstance->PianoMode();
}

void DrumModeCallback(UIButton* button) {
    _currentInstance->DrumMode();
}

void SynthModeCallback(UIButton* button) {
    _currentInstance->SynthMode();
}

void PlayAllCallback(UIButton* button) {
    _currentInstance->PlayAll();
}

void PlayCurrentCallback(UIButton* button) {
    _currentInstance->PlayCurrent();
}

void ResetCallback(UIButton* button) {
    _currentInstance->Reset();
}

void RecordCallback(UIButton* button) {
    _currentInstance->Record();
}

void drawPlayButton() {
    glBegin(GL_TRIANGLES);
    glVertex2f(PLAY_SIZE, PLAY_SIZE/2.0f);
    glVertex2f(0.0f, PLAY_SIZE);
    glVertex2f(0.0f, 0.0f);
    glEnd();
}

void Interface::drawTopBar() { 
    glBegin(GL_LINES);
        glColor3f(0.0f, 0.0f, 0.0f); 
        glVertex2f(0.0f, _winHeight-_topBarHeight);
        glVertex2f(_winWidth, _winHeight-_topBarHeight);
    glEnd();
    
    glPushMatrix();
    glTranslatef((_winWidth + _symbolGap + PLAY_BUTTON_SIZE - PLAY_SIZE)/2.0f, _winHeight - (_symbolSize + PLAY_SIZE)/2.0f - _symbolGap, 0.0f);
    drawPlayButton();

    glTranslatef(-_symbolGap - PLAY_BUTTON_SIZE, 0.0f, 0.0f);
    drawPlayButton();
   
    glTranslatef(0.0f, -4.0f, 0.0f);
    glRectf(0.0f, 0.0f, PLAY_SIZE, 4.0f);
    glPopMatrix();
    
    if (_isRecording) {
        glPushMatrix();
        glColor3f(1.0f, 1.0f, 1.0f);
        glTranslatef(_winWidth/2.0f - _symbolGap - PLAY_BUTTON_SIZE - _symbolSize/2.0f - _symbolSize/4.0f, _winHeight - _symbolGap - _symbolSize/2.0f - _symbolSize/4.0f, 0.0f);
        glRectf(0.0f, 0.0f, _symbolSize/2.0f, _symbolSize/2.0f);
        glPopMatrix();
    }
 }

#pragma mark -

#pragma mark - Keyboard
bool isUpperCase(char note) {
    return (note <= 90);
}

void Interface::toggleKeyHighlight(int index, bool on) {
    _keyboard[index - (NUM_KEYS * FIRST_OCTAVE)].highlighted = on;
}

/*Draws the currKey using the specified color. Expects the bottom left corner*/
void Interface::drawKey(Key currKey, STColor3f color, int x, int y) {
    glPushMatrix();
    glTranslatef(x, y, 0.0f);
    
    if (currKey.highlighted) {
        glColor3f(0.0f, 1.0f, 0.0f);
    } else {
        glColor3f(color.r, color.g, color.b);
    }
    
    float keyWidth = (currKey.whiteKey) ? _wKeyWidth : _bKeyWidth;
    float keyHeight = (currKey.whiteKey) ? _wKeyHeight : _bKeyHeight;
    
    if (!currKey.whiteKey) {
        glTranslatef(0.0f, 0.0f, 1.0f);
    }
    
    glRectf(0.0f, 0.0f, keyHeight, keyWidth);
    
    if (currKey.whiteKey) {
        glColor3f(0.0f, 0.0f, 0.0f);
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
        glRectf(0.0f, 0.0f, keyHeight, keyWidth);
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    }
    
    glPopMatrix();
}

/*Iterates through the keyboard and draws the keys*/
void Interface::drawKeyboard() {
    glPushMatrix();
    glTranslatef(0.0f, -_topBarHeight - 1.0f, 0.0f);
    int numWhiteKey = 0;
    for (int i = 0; i < NUM_KEYS * NUM_OCTAVES; i++) {
        Key currKey = _keyboard[i];
        STColor3f color(currKey.whiteKey);
        currKey.x = (!currKey.whiteKey) * 0.5f * (KEY_HEIGHT);
        
        float keyY = numWhiteKey * _wKeyWidth;
        if (currKey.whiteKey) {
            numWhiteKey++;
            keyY += _wKeyWidth;
        } else {
            keyY += _bKeyWidth * 0.5f;
        }
        currKey.y = _winHeight - keyY;
        
        drawKey(currKey, color, currKey.x, currKey.y);
        _keyboard[i] = currKey;
    }
    glPopMatrix();
}

void Interface::createKey(char note, int index) {    
    for (int i = 0 ; i < NUM_OCTAVES; i++) {
        Key newKey;
        newKey.note = note;
        newKey.octave = i + FIRST_OCTAVE;
        newKey.whiteKey = isUpperCase(note);
        newKey.highlighted = false;
        _keyboard[index + (i * NUM_KEYS)] = newKey;
    }
}

/*Creates all key objects.*/
void Interface::createKeyboard() {
    createKey('C', 0);
    createKey('c', 1);
    createKey('D', 2);
    createKey('d', 3);
    createKey('E', 4);
    createKey('F', 5);
    createKey('f', 6);
    createKey('G', 7);
    createKey('g', 8);
    createKey('A', 9);
    createKey('a', 10);
    createKey('B', 11);
}
#pragma mark -

#pragma mark - Reverb
void Interface::drawReverbBox() {
    //Top Line
    glBegin(GL_LINES);
        glColor3f(0.0f, 0.0f, 0.0f);
        glVertex2f(0.0f, _rBoxHeight - 1.0f);
        glVertex2f(_winWidth, _rBoxHeight - 1.0f);
    glEnd();
}

void CaptureMouse(UIButton *button) {
    _clickedButton = button;
}

/*Clicked Widget stops getting all mouse events*/
void ReleaseMouse() {
    _clickedButton = NULL;
}

/*Tells application that a widget is being dragged. Gets the location of where the drag starts*/
void BeginDrag(UIButton *button, STPoint2 m) {
    _draggedButton = button;
    _mouseDragStart = m;
}

STPoint2 RotatePoint(STPoint2 p, float deg) {
    STTransform4 rotate = STTransform4::Rotation(0.0f, 0.0f, -deg * M_PI / 180.0f);
    STPoint3 newP;
    newP.x = p.x; newP.y = p.y; newP.z = 1.0f;
    newP = rotate * newP;
    return STPoint2(newP.x, newP.y);
}

/*Stop dragging widget*/
void EndDrag() {
    _draggedButton = NULL;
    _mouseDragStart.x = _mouseDragStart.y = 0;
}

void Interface::mouseClick(int button, int state, int x, int y) {
    STPoint2 m(x, _winHeight - y);
    if (state == GLUT_UP) {
        if (_clickedButton) {
            _clickedButton->HandleMouseUp(m);
        }
        _screenDrag = false;
    } else if (state == GLUT_DOWN) {
        if (screenIsHit(m)) {
            BeginDrag(NULL, m);
            _screenDrag = true;
        } else {
            for (int i = 0; i < _buttons.size(); i++) {
                UIButton* currButton = _buttons.at(i);
                if (currButton->HitTest(m)) {
                    currButton->HandleMouseDown(m);
                    if (currButton == _revButton) {
                        _rotate = (button == GLUT_RIGHT_BUTTON);
                    }
                    break;
                }
            }
        }
    }
}

void mouseClickCallBack(int button, int state, int x, int y) {
    _currentInstance->mouseClick(button, state, x, y);
}

bool Interface::rButtonInBounds(float dx, float dy) {
    UIRectangle rect = _revButton->GetRectangle();
    return (rect.pMin.x + dx > 0 &&
            rect.pMin.y + dy > 0 &&
            rect.pMax.x + dx < _winWidth &&
            rect.pMax.y + dy < _rBoxHeight);
}

void Interface::mouseDrag(int x, int y) { 
    STPoint2 m(x, _winHeight - y);
    float diffX = m.x - _mouseDragStart.x;
    float diffY = m.y - _mouseDragStart.y;
    
    if (_screenDrag) {
        if (_beatScreen[_currentMode].translationOffset - diffX >= 0) {
            _beatScreen[_currentMode].translationOffset -= diffX;
            _mouseDragStart = m;
        }
    } else if (_draggedButton && _draggedButton == _revButton) {
        if (_rotate) {
            float rotateDiff = (diffX + diffY) * 360.0f / _rBoxWidth;
            if (_rRotate + rotateDiff >= 0 && _rRotate + rotateDiff <= 360.0f) {
                _rRotate += rotateDiff;
                _rWidth = _rRotate * MAX_RWIDTH / 360.0f;
            }
        } else if (rButtonInBounds(diffX, diffY)) {
            _draggedButton->MoveBy(diffX, diffY);
            _mouseDragStart = m;
            _rX += diffX;
            _rY += diffY;
            _rRoomSize = _rX * MAX_RROOMSIZE / (_rBoxWidth - _rSize);
            _rDamp = _rY * MAX_RDAMP / (_rBoxHeight - _rSize);
        }
        
        fluid_synth_set_reverb(_fsynth, _rRoomSize, _rDamp, _rWidth, _rLevel);
        
    } else {
        mouseMove(x, y);
    }
    glutPostRedisplay();
}

void mouseDragCallBack(int x, int y) { 
    _currentInstance->mouseDrag(x, y);
}

void Interface::mouseMove(int x, int y) {
    STPoint2 m(x, _winHeight - y);
    if (_topButton && !_topButton->HitTest(m)) { //If mouse was over button, but not anymore
        _topButton->HandleMouseLeave(); //mouse leave
        _topButton = NULL; 
        glutPostRedisplay();
    }
    
    for (int i = 0 ; i < _buttons.size(); i++) {
        UIButton* currButton = _buttons.at(i);
        if (!_clickedButton || _clickedButton == currButton) { //Mouse isn't over button or mouse hasn't left button
            if (currButton->HitTest(m)) { //If mouse over revButton
                currButton->HandleMouseMove(m); 
                if (!_topButton) { //If mouse just went over button
                    currButton->HandleMouseEnter(); //mouse enter
                    _topButton = currButton;
                    glutPostRedisplay();
                }
                break;
            }
        }
    }
    
}

void mouseMoveCallBack(int x, int y) {
    _currentInstance->mouseMove(x, y);
}

void Interface::special(int key, int x, int y) {
    switch (key) {
        case GLUT_KEY_UP:
            if (_rLevel < MAX_RLEVEL) {
                _rLevel += 0.1f;
                _revButton->SetColor(STColor3f(_rLevel));
            }
            break;
        case GLUT_KEY_DOWN:
            if (_rLevel > 0.0f) {
                _rLevel -= 0.1f;
                _revButton->SetColor(STColor3f(_rLevel));
            }
            break; 
        default:
            break;
    }
    fluid_synth_set_reverb(_fsynth, _rRoomSize, _rDamp, _rWidth, _rLevel);
    glutPostRedisplay();
}

void specialCallBack(int key, int x, int y) {
    _currentInstance->special(key, x, y);
}
#pragma mark -

#pragma mark - Callbacks
void Interface::drawBackground() {
//    glBegin(GL_TRIANGLE_STRIP);
//    glColor3f(1.0f, 1.0f, 1.0f);
//    glVertex2f(_winWidth * 0.5f, _winHeight * 0.75f);
//    glVertex2f(_winWidth * 0.25f, _winHeight * 0.5f);
//    glVertex2f(_winWidth * 0.75f, _winHeight * 0.5f);
//    glVertex2f(_winWidth * 0.5f, _winHeight * 0.25f);
//    glEnd();
}

/*Display callback*/
void Interface::display() {
    // Clear the window.
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    
    glLineWidth(2.0f);
    drawBackground();
    drawKeyboard();
    drawCurrentScreen();
    if (_isPlayingBack) {
        drawCursor();
    }
    drawReverbBox();
    drawTopBar();
    
    for (int i = 0 ; i < _buttons.size(); i++) {
        if (_buttons.at(i) == _revButton) {
            glPushMatrix();
            glTranslatef(_rX, _rY, 0.0f);
            glRotatef(_rRotate, 0.0f, 0.0f, -1.0f);
            glTranslatef(-_rX, -_rY, 0.0f);
            _buttons.at(i)->Display();
            glPopMatrix();
        } else {
            _buttons.at(i)->Display();
        }
    }
    
    glutSwapBuffers();
}

void displayCallBack() {
    _currentInstance->display();
}

/*Reshape callback*/
void Interface::reshape(int w, int h) {
    _winWidth = w;
    _winHeight = h;
    
    _keyboardLength = 0.75f * h;
    _wKeyWidth = (_keyboardLength/((float)NUM_WHITE_KEYS * NUM_OCTAVES));
    _bKeyWidth = _wKeyWidth / 2.0f;
    
    _rBoxWidth = _winWidth;
    _rBoxHeight = _winHeight - _topBarHeight - _keyboardLength;
    _rSize = (float)_winWidth * 0.25f * 0.25f;
    _revButton->SetRectangle(UIRectangle(STPoint2(_rX, _rY), STPoint2(_rSize + _rX, _rSize + _rY)));
    float startX = _winWidth - 4 * ( _symbolSize + _symbolGap);
    float startY = _winHeight - _symbolSize - _symbolGap;
    UIRectangle rect = UIRectangle(STPoint2(startX, startY), STPoint2(startX + _symbolSize, startY + _symbolSize));
   
    _dButton->SetRectangle(rect);
    
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    _sButton->SetRectangle(rect);
    
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    _gButton->SetRectangle(rect);
    
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    _pButton->SetRectangle(rect);
    
    rect.pMin.x = _winWidth/2.0f - _symbolGap - PLAY_BUTTON_SIZE - _symbolSize;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    _recordButton->SetRectangle(rect);
    
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
    _playAllButton->SetRectangle(rect);
    
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
   _playCurrentButton->SetRectangle(rect);

    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
    _resetButton->SetRectangle(rect);
    
    rect.pMin.x = 1.0f;
    rect.pMax.x = 25.0f;
    
    _closeButton->SetRectangle(rect);
    
    
    glViewport(0, 0, _winWidth, _winHeight);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glOrtho(0.0f, w, 0.0f, h, -1.0f, 1.0f);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void reshapeCallBack(int w, int h) {
    _currentInstance->reshape(w, h);
}

#pragma mark -

void Close(UIButton *button) {
    Exit(); 
}

void Interface::AddButton(UIButton *button, UIRectangle rect) {
    button->SetRectangle(rect);
    _buttons.push_back(button);
}

void Interface::createButtons() {
    //Reverb Button
    STFont* stdFont = new STFont("arial.ttf", 24);
    STColor3f red(1.0f, 0.0f, 0.0f);
    
    _revButton = new UIButton("", stdFont, STColor3f(), STColor3f(_rLevel));
    UIRectangle rect(STPoint2(_rX, _rY), STPoint2(_rSize + _rX, _rSize + _rY));
    AddButton(_revButton, rect);
        
    //Top Buttons
    float startX = _winWidth - 4 * ( _symbolSize + _symbolGap);
    float startY = _winHeight - _symbolSize - _symbolGap;

    _dButton = new UIButton("D", stdFont, STColor3f(0.0f), TOP_BUTTON_COLOR, &DrumModeCallback);
    rect = UIRectangle(STPoint2(startX, startY), STPoint2(startX + _symbolSize, startY + _symbolSize));
    AddButton(_dButton, rect);
    
    _sButton = new UIButton("S", stdFont, STColor3f(0.0f), TOP_BUTTON_COLOR, &SynthModeCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    AddButton(_sButton, rect);
    
    _gButton = new UIButton("G", stdFont, STColor3f(0.0f), TOP_BUTTON_COLOR, &GuitarModeCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    AddButton(_gButton, rect);
    
    _pButton = new UIButton("P", stdFont, STColor3f(0.0f), SELECTED_BUTTON_COLOR, &PianoModeCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    AddButton(_pButton, rect);
    
    _recordButton = new UIButton("", stdFont, STColor3f(), red, &RecordCallback);
    rect.pMin.x = _winWidth/2.0f - _symbolGap - PLAY_BUTTON_SIZE - _symbolSize;
    rect.pMax.x = rect.pMin.x + _symbolSize;
    AddButton(_recordButton, rect);
    
    _playAllButton = new UIButton("", stdFont, STColor3f(), TOP_BUTTON_COLOR, &PlayAllCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
    AddButton(_playAllButton, rect);
    
    _playCurrentButton = new UIButton("", stdFont, STColor3f(), TOP_BUTTON_COLOR, &PlayCurrentCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
    AddButton(_playCurrentButton, rect);
    
    _resetButton = new UIButton("RESET", new STFont("arial.ttf", 14), STColor3f(0.75f), STColor3f(0.0f), &ResetCallback);
    rect.pMin.x = rect.pMax.x + _symbolGap;
    rect.pMax.x = rect.pMin.x + PLAY_BUTTON_SIZE;
    AddButton(_resetButton, rect);
    
    _closeButton = new UIButton("X", new STFont("arial.ttf", 16), STColor3f(1.0f), red, &Close);
    rect.pMin.x = 1.0f;
    rect.pMax.x = 25.0f;
    AddButton(_closeButton, rect);
}

void Interface::init(int argc, char ** argv, fluid_synth_t * fsynth) {
    _currentInstance = this;
    _fsynth = fsynth;
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowSize(_winWidth, _winHeight);
    
    glutCreateWindow("Lead Singer");
    
    glutDisplayFunc(displayCallBack);
    glutReshapeFunc(reshapeCallBack);
    glutSpecialFunc(specialCallBack);
    glutMouseFunc(mouseClickCallBack);
    glutMotionFunc(mouseDragCallBack);
    glutPassiveMotionFunc(mouseMoveCallBack);
    glutIdleFunc(displayCallBack);
    
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);    
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_ALPHA_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    createKeyboard();
    
    createButtons();
    
    glutMainLoop();
}

