//
//  Interface.h
//  Lead Singer
//
//  Created by Alvin Heng on 12/4/11.
//  Copyright 2011 Stanford University. All rights reserved.
//

#ifndef Lead_Singer_Interface_h
#define Lead_Singer_Interface_h

#include "Thread.h"
#include "include/fluidsynth.h"
#include "UIButton.h"

#define NUM_KEYS 12
#define NUM_OCTAVES 5
#define FIRST_OCTAVE 2
#define NUM_WHITE_KEYS 7
#define KEY_HEIGHT 40.0f

#define MAX_RROOMSIZE 1.2f
#define MAX_RDAMP 1.0f
#define MAX_RWIDTH 100.0f
#define MAX_RLEVEL 1.0f

#define NUM_NOTES_ON_SCREEN 1024
#define NUM_MODES 4

#define PLAY_BUTTON_SIZE 50
#define PLAY_SIZE 15.0f
#define TOP_BUTTON_COLOR STColor3f(0.75f, 0.75f, 0.75f)
#define SELECTED_BUTTON_COLOR STColor3f(0.25f, 0.25f, 0.25f)

struct Key {
    char note;
    int noteIndex;
    int octave;
    bool whiteKey;
    bool highlighted;
    float x, y;
    float duration;
    bool doneAnimating;
    float timer;
};

struct Screen {
    Key beats[NUM_NOTES_ON_SCREEN];
    int numBeats;
    float translationOffset;
};

enum SynthMode {
    GUITARMODE = 0,
    PIANOMODE = 1,
    DRUMMODE = 2,
    SYNTHMODE = 3
};

class Interface {

public:
    Interface();
    ~Interface();
    void init(int argc, char ** argv, fluid_synth_t * fsynth);
    
    void display();
    void reshape(int w, int h);
    void special(int key, int x, int y);
    void mouseClick(int button, int state, int x, int y);
    void mouseDrag(int x, int y);
    void mouseMove(int x, int y);
    void GuitarMode();
    void PianoMode();
    void DrumMode();
    void SynthMode();
    void PlayAll();
    void PlayCurrent();
    void Record();
    void Reset();
    
    void toggleKeyHighlight(int index, bool on);
    
    void addBeat(int noteIndex, char note);
    void extendBeat();
    
private:
    //Window Dimensions
    int _winWidth;
    int _winHeight;
    
    //Keyboard
    Key _keyboard[NUM_KEYS * NUM_OCTAVES];
    void createKeyboard();
    void createKey(char note, int index);
    void drawKeyboard();
    void drawKey(Key currKey, STColor3f color, int x, int y);
    float _keyboardLength;
    float _wKeyWidth;
    float _wKeyHeight;
    float _bKeyWidth;
    float _bKeyHeight;
    
    //Reverb
    void drawReverbBox();
    bool rButtonInBounds(float dx, float dy);
    UIButton* _revButton;
    float _rBoxWidth;
    float _rBoxHeight;
    float _rRoomSize;
    float _rDamp;
    float _rWidth;
    float _rLevel;
    float _rX;
    float _rY;
    float _rRotate;
    float _rSize;
    
    //Piano Screen
    Screen _beatScreen[NUM_MODES];
    void drawBeat(Key key);
    void drawCurrentScreen();
    bool _isPlayingBack;
    bool screenIsHit(STPoint2 m);
    bool _screenDrag;
    
    //Top Toolbar
    UIButton* _gButton;
    UIButton* _pButton;
    UIButton* _dButton;
    UIButton* _sButton;
    UIButton* _playAllButton;
    UIButton* _playCurrentButton;
    UIButton* _resetButton;
    UIButton* _closeButton;
    UIButton* _recordButton;
    void drawTopBar();
    float _topBarHeight;
    float _symbolSize;
    float _symbolGap;
    enum SynthMode _currentMode;
    bool _isRecording;
    
    
    void createButtons();
    void AddButton(UIButton *button, UIRectangle rect);
    
    fluid_synth_t * _fsynth; 
    std::vector<UIButton*> _buttons;
    void drawBackground();
    void drawCursor();    
};

#endif
