// UIButton.cpp
#include "UIButton.h"
#include <iostream>

using namespace std;

void CaptureMouse(UIButton *button);
void ReleaseMouse();
void BeginDrag(UIButton *widget, STPoint2 m);
void EndDrag();

UIButton::UIButton() {
}

UIButton::UIButton(const std::string& text, STFont* font, STColor3f fontColor, STColor3f bgColor, FireCallback callback)
: _text(text), _font(font), _fontColor(fontColor), _bgColor(bgColor), _callback(callback) {
    isSelected = false;
    isHighlighted = false;
}

/*Prints state*/
void UIButton::printState(char *action) {
    printf("%s. isHighlighted: %u isSelected: %u\n", action, isHighlighted, isSelected);
}

/*Handles mouse events*/ 
void UIButton::HandleMouseDown(const STPoint2& position) {
    CaptureMouse(this);
    BeginDrag(this, position);
    isSelected = true;
    isHighlighted = true;
    printState((char*)"Button Down");
}

void UIButton::HandleMouseUp(const STPoint2& position) {
    ReleaseMouse();
    EndDrag();
    if (isHighlighted && isSelected) {
        if (_callback) {
            _callback(this);
            printf("FIRE!\n");
        }
    } else {
        isHighlighted = false;
    }
    isSelected = false;
    printState((char*)"Button Up");
}

void UIButton::HandleMouseEnter() {
    if (isHighlighted) {
        isSelected = true;
    } else {
        isHighlighted = true;
    }
    printState((char*)"Enter");
}

void UIButton::HandleMouseLeave() {
    if (!isSelected) {
        isHighlighted = false;
    } else {
        isSelected = false;
    }
    printState((char*)"Exit");
}

void UIButton::HandleMouseMove(const STPoint2& position) {
    printf("Mouse at: (%f, %f)\n", position.x, position.y);
}

void UIButton::MoveBy(int dx, int dy) {
    STPoint2 pMin = _rectangle.pMin;
    STPoint2 pMax = _rectangle.pMax;
    pMin.x += dx; pMax.x += dx;
    pMin.y += dy; pMax.y += dy;
    
    SetRectangle(UIRectangle(pMin, pMax));
}

bool UIButton::HitTest(const STPoint2& position)
{
    return GetRectangle().Contains(position);
}

//
// Display the button to the current OpenGL context.
//
void UIButton::Display() {
    UIRectangle r = GetRectangle();
    STColor3f bgColor = _bgColor;
    if (isSelected) {
        bgColor = STColor3f(0.25f);
    }
    GLfloat width = r.pMax.x - r.pMin.x;
    GLfloat height = r.pMax.y - r.pMin.y;
    
    glPushMatrix();
    glTranslatef(r.pMin.x, r.pMin.y, 0.0f);
    //outline
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glColor3f(0.0f, 0.0f, 0.0f);
    glRectf(-0.25f, -0.25f, width+0.25f, height+0.25f);
    //inside
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glColor4f(bgColor.r, bgColor.g, bgColor.b, 1.0f);
    glRectf(0.0f, 0.0f, width, height);
    if (_font) {
        glTranslatef((width - _font->ComputeWidth(_text))/2, (height - _font->GetAscender())/2, 1.0f);
        _font->DrawString(_text, STColor4f(_fontColor));
    }
    glPopMatrix();
}
