// UIButton.h
#ifndef __UIBUTTON_H__
#define __UIBUTTON_H__

#include "UIRectangle.h"
#include <GLUT/glut.h>

class UIButton {
    
public:
    // Type of callbacks to handle button-press "fire" events.
    typedef void (*FireCallback)(UIButton* button);
    
    UIButton();
    //
    // Constructor: Initialize a UI button with
    // a given font face and label text.
    //
    UIButton(const std::string& text, STFont* font, STColor3f fontColor, STColor3f bgColor, FireCallback callback = NULL);
    
    void HandleMouseDown(const STPoint2& position);
    void HandleMouseUp(const STPoint2& position);
    void HandleMouseEnter();
    void HandleMouseLeave();
    void HandleMouseMove(const STPoint2& position);
    
    bool HitTest(const STPoint2& position);
    
    void MoveBy(int x, int y);
    
    UIRectangle GetRectangle() const { return _rectangle; }
    void SetRectangle(const UIRectangle& rectangle) { _rectangle = rectangle; }
    void SetColor(STColor3f color) { _bgColor = color; }
    void Display();
    
    bool isSelected;
    bool isHighlighted; 
    
private:
    STFont* _font;
    std::string _text;
    
    STColor3f _bgColor;
    STColor3f _fontColor;
    
    // "Fire" event callback
    FireCallback _callback;
    
    UIRectangle _rectangle;
    
    void printState(char* action);

};

#endif // __UIBUTTON_H__
