//
//  main.cpp
//  Lead Singer
//
//  Created by Alvin Heng on 11/15/11.
//  Copyright 2011 Stanford University. All rights reserved.
//

#include <iostream>
#include <math.h>
#include "RtAudio.h"
#include "Interface.h"
#include "xtract/libxtract.h"
#include "/opt/local/include/sndfile.h"


#define SAMPLE float
#define FORMAT RTAUDIO_FLOAT32
#define SAMPLE_RATE 44100
#define NUM_KEYS 12
#define NUM_CHANNELS 2

#define C0 16.351f
#define c0 17.324f
#define D0 18.354f
#define d0 19.445f
#define E0 20.601f
#define F0 21.827f
#define f0 23.124f
#define G0 24.499f
#define g0 25.956f
#define A0 27.50f
#define a0 29.135f
#define B0 30.868f

#define FIRST_KEY 28
#define LAST_KEY 72

#define MAX_VEL 127.0f
#define MAX_RMS 0.1f

#define AVG_NOTE_DURATION 200000

#define KICK_INDEX 36;
#define SNARE_INDEX 39;

Interface _interface;
fluid_synth_t * _fsynth; 
float _frequencies[NUM_KEYS] = {C0, c0, D0, d0, E0, F0, f0, G0, g0, A0, a0, B0};
int _lastKeyOn = 0;
bool _playingBack = false;
bool _playingAll = false;
bool _isRecording = false;
SNDFILE* _outfiles[NUM_MODES] = {NULL, NULL, NULL, NULL};
SNDFILE* _outfile = NULL;
SynthMode _currMode = PIANOMODE;
SNDFILE* _infiles[NUM_MODES] = {NULL, NULL, NULL, NULL};

using namespace std;

void Exit() {
    for (int i = 0 ; i <NUM_MODES; i++) {
        sf_write_sync(_outfiles[_currMode]);
        sf_write_sync(_outfile);
        sf_close(_outfiles[i]);
        sf_close(_outfile);
        sf_close(_infiles[i]);
    }
    exit(1);
}

void ChangeMode(SynthMode currMode) {
    _currMode = currMode;
}

void ResetOutfile() {
    SNDFILE* outfile = _outfiles[_currMode];    
    if (outfile) {
        sf_close(outfile);
    }
    SF_INFO sfinfo;
    sfinfo.channels = NUM_CHANNELS;
    sfinfo.samplerate = SAMPLE_RATE;
    sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
    char outName[15] = "recording .wav";
    switch (_currMode) {
        case PIANOMODE:
            outName[9] = 'P';
            break;
        case GUITARMODE:
            outName[9] = 'G';
            break;
        case DRUMMODE:
            outName[9] = 'D';
            break;
        case SYNTHMODE:
            outName[9] = 'S';
            break;
        default:
            break;
    }
    outfile = sf_open(outName, SFM_WRITE, &sfinfo);
    _outfile = sf_open("recordingCombo.wav", SFM_WRITE, &sfinfo);
    if (!outfile || !_outfile) exit(1);
    _outfiles[_currMode] = outfile;
}

void startStopRec() {
    _isRecording = !_isRecording;
    if (!_isRecording) {
        sf_write_sync(_outfiles[_currMode]);
    }
}

/*Called by Interface.*/
void playback() {
    _playingBack = true;
    SF_INFO sfinfo;
    char inName[15] = "recording .wav";
    switch (_currMode) {
        case PIANOMODE:
            inName[9] = 'P';
            break;
        case GUITARMODE:
            inName[9] = 'G';
            break;
        case DRUMMODE:
            inName[9] = 'D';
            break;
        case SYNTHMODE:
            inName[9] = 'S';
            break;
        default:
            break;
    }

    if (!(_infiles[_currMode] = sf_open(inName, SFM_READ, &sfinfo))) {
        cout<<"Not able to open input file "<<inName<<". \n";
        exit(1);
    }
}

void playbackAll() {
    _playingBack = true;
    _playingAll = true;
    for (int i = 0 ; i < NUM_MODES; i++) {
        SF_INFO sfinfo;
        char inName[15] = "recording .wav";
        switch (i) {
            case PIANOMODE:
                inName[9] = 'P';
                break;
            case GUITARMODE:
                inName[9] = 'G';
                break;
            case DRUMMODE:
                inName[9] = 'D';
                break;
            case SYNTHMODE:
                inName[9] = 'S';
                break;
            default:
                break;
        }
        if (!(_infiles[i] = sf_open(inName, SFM_READ, &sfinfo))) {
            cout<<"Not able to open input file "<<inName<<". \n";
            exit(1);
        }

    }    
}

/*Takes a frequency and clamps it to the frequency of a note.*/
float clampFreq(float freq) {
    int octave = 0;
    while (freq > C0 * 2.0f) { //Divide by 2 until its in the range of [C0, B0]
        freq /= 2;
        octave++; //1 octave for every division
    }
       
    float minDiff = fabsf(_frequencies[0] - freq); //min diff between given freq and defined note constants
    int indexOfMinDiff = 0;
    for (int i = 1; i < NUM_KEYS; i++) {
        float prevDiff = minDiff; //keep track of current minDiff
        minDiff = min(minDiff, fabsf(_frequencies[i] - freq)); //find min diff
        if (prevDiff != minDiff) { //if current minDiff changed, indexOfMinDiff is index of new minimum
            indexOfMinDiff = i;
        }
    }
    freq = _frequencies[indexOfMinDiff]; //frequency = defined note constant at indexOfMinDiff
    for (int i = 0; i < octave; i++) { //Multiply by 2 for each octave
        freq *= 2;
    }
   
    return freq;
}

/* 
 * note = log2(freq / 440.0f) * 12 + 69.
 * A4 is 440.0f Hz. Corresponds to index 69.
 * Frequency must be clamped to defined note constants
 */
int getNoteIndexFromFrequency(float freq) {
    int index = log2f(freq/440.0f) * 12.0f + 69.1f;
    if (_currMode == DRUMMODE) {
        if (index < 50) {
            return KICK_INDEX;
        } else {
            return SNARE_INDEX;
        }
    }
    return index;
}

/*Return note from given index. Currently only used for debugging purposes*/
char getNoteFromNoteIndex(int index) {
    switch ((index%72)%12) {
        case 0:
            return 'C';
        case 1:
            return 'c';
        case 2:
            return 'D';
        case 3:
            return 'd';
        case 4:
            return 'E';
        case 5:
            return 'F';
        case 6:
            return 'f';
        case 7:
            return 'G';
        case 8:
            return 'g';
        case 9:
            return 'A';
        case 10:
            return 'a';
        case 11:
            return 'B';
        default:
            return 'X';
            break;
    }
}
int testnote = FIRST_KEY;
int audioCallback(void * outputBuffer, void * inputBuffer, unsigned int numFrames,
                  double streamTime, RtAudioStreamStatus status, void * data) {
    
    SAMPLE* output = (SAMPLE* )outputBuffer;
    SAMPLE* input = (SAMPLE* )inputBuffer;
    
    if (!_playingBack) {
    
        //Get the frequency from the input buffer and clamp it to the frequency of a note
        SAMPLE frequency;
        SAMPLE rate = SAMPLE_RATE;
        xtract_init_fft(numFrames, XTRACT_SPECTRUM);
        xtract_failsafe_f0(input, numFrames, &rate, &frequency);
        if (frequency > 0) {
            frequency = clampFreq(frequency);
        }
        
        //Get the rmsAmplitude
        SAMPLE rmsAmplitude;
        xtract[XTRACT_RMS_AMPLITUDE](input, numFrames, NULL, &rmsAmplitude);
        
        if (frequency > 0 && rmsAmplitude > 0.02f) {
            int noteIndex = getNoteIndexFromFrequency(frequency);
            if (noteIndex >= FIRST_KEY && noteIndex <= LAST_KEY) {
                if (_lastKeyOn != noteIndex) {
                    fluid_synth_noteoff(_fsynth, 1, _lastKeyOn);
                    fluid_synth_noteon(_fsynth, 1, noteIndex, fminf(rmsAmplitude * MAX_VEL / MAX_RMS, MAX_VEL));
                    _interface.toggleKeyHighlight(_lastKeyOn, false);
                    _lastKeyOn = noteIndex;
                    _interface.toggleKeyHighlight(_lastKeyOn, true);
                    
                    _interface.addBeat(noteIndex, getNoteFromNoteIndex(noteIndex));
                } 
            }
            //  cout<<getNoteFromNoteIndex(noteIndex)<<" - "<<noteIndex<<endl;
        } else if (rmsAmplitude < 0.02f) { //note should be turned off
            _interface.toggleKeyHighlight(_lastKeyOn, false);
            fluid_synth_noteoff(_fsynth, 1, _lastKeyOn);
            _lastKeyOn = 0;
        } else if (_lastKeyOn != 0){
            _interface.extendBeat();
        }
        fluid_synth_write_float(_fsynth, numFrames, output, 0, NUM_CHANNELS, output, 1, NUM_CHANNELS);
        if (_isRecording) {
            sf_write_float(_outfiles[_currMode], output, numFrames * NUM_CHANNELS);
        }
    } else if (_playingAll) {
        SAMPLE temp[numFrames * NUM_CHANNELS];
        SAMPLE temp2[numFrames * NUM_CHANNELS];
        SAMPLE temp3[numFrames * NUM_CHANNELS];
        SAMPLE temp4[numFrames * NUM_CHANNELS];
        sf_count_t count = sf_read_float(_infiles[0], temp, numFrames * NUM_CHANNELS);
        count += sf_read_float(_infiles[1], temp2, numFrames * NUM_CHANNELS);
        count += sf_read_float(_infiles[2], temp3, numFrames * NUM_CHANNELS);
        count += sf_read_float(_infiles[3], temp4, numFrames * NUM_CHANNELS);

        for (int i = 0 ; i < numFrames * NUM_CHANNELS; i++) {
            output[i] = temp[i] + temp2[i] + temp3[i] + temp4[i];
        }
        
        sf_write_float(_outfile, output, numFrames * NUM_CHANNELS);
        if (count == 0) {
            _playingBack = false;   
        }
    } else { //playing back
        sf_count_t count = sf_read_float(_infiles[_currMode], output, numFrames * NUM_CHANNELS);
        if (count == 0) {
            _playingBack = false;   
        }
    }
    
    return 0;
}

int main(int argc, char ** argv) {
    // set up fluid synth stuff
    fluid_settings_t * settings = new_fluid_settings(); 
    _fsynth = new_fluid_synth(settings);
    
    if (fluid_synth_sfload(_fsynth, "piano.sf2", 1) == -1) {
        cerr << "Error loading sound font" << endl;
        exit(1);
    }
    
    _currMode = DRUMMODE;
    ResetOutfile();
    _currMode = SYNTHMODE;
    ResetOutfile();
    _currMode = GUITARMODE;
    ResetOutfile();
    _currMode = PIANOMODE;
    ResetOutfile();
    
    
    RtAudio* audio = NULL;
    unsigned int bufferSize = 1024;
   
    try {
        audio = new RtAudio();
        cerr << "buffer size: " << bufferSize << endl;
    }
    catch(RtError &err) {
        err.printMessage();
        exit(1);
    }
    
    if( audio->getDeviceCount() < 1) {
        cout << "no audio devices found!" << endl;
        exit(1);
    }
    
    audio->showWarnings(true);
    
    // set input and output parameters
    RtAudio::StreamParameters iParams, oParams;
    iParams.deviceId = audio->getDefaultInputDevice();
    iParams.nChannels = 1;
    iParams.firstChannel = 0;
    oParams.deviceId = audio->getDefaultOutputDevice();
    oParams.nChannels = NUM_CHANNELS;
    oParams.firstChannel = 0;
    
    // create stream options
    RtAudio::StreamOptions options;
    
    try {
        audio->openStream(&oParams, &iParams, FORMAT, SAMPLE_RATE, &bufferSize, &audioCallback, NULL, &options);
        audio->startStream();
        
        // test RtAudio functionality for reporting latency.
        cout << "stream latency: " << audio->getStreamLatency() << " frames" << endl;
        
        //Give main thread to GL
        _interface.init(argc, argv, _fsynth);
    }
    catch(RtError &err) {
        err.printMessage();
        goto cleanup;
    }
    
    // if we get here, stop!
    try {
        audio->stopStream();
    }
    catch(RtError & err) {
        err.printMessage();
    }
    
    // Clean up
cleanup:
    if(audio) {
        audio->closeStream();
        delete audio;
    }    
    
    return 0;
}
